local AddonName, a = ...
local L = a.Localize
local s = SpellFlashAddon
local c = BittensSpellFlashLibrary

c.Init(a)

------------------------------------------------------------------------ Common
local markDebuffs = {
    1130, -- Hunter's Mark
    88691, -- Marked for Death
}

a.spells["Mend Pet"] = {
    ID = 136,
    FlashColor = "red",
    CheckFirst = function()
        return s.HealthPercent("pet") < 80
    end
}

a.spells["Hunter's Mark"] = {
    ID = 1130,
    Debuff = markDebuffs,
    FlashColor = "yellow",
}

a.spells["Serpent Sting"] = {
    ID = 1978,
    Continue = 1,
    FlashColor = "yellow",
    NoPowerCheck = 1,
    EvenIfNotUsable = 1,
    CheckFirst = function(z)
        return a.Focus >= s.SpellCost(z.ID)
            and c.ShouldCastToRefresh(
                "Serpent Sting", "Serpent Sting", 2, true)
    end
}

a.spells["Kill Shot"] = {
    ID = 53351,
}

a.spells["Arcane Shot"] = {
    ID = 3044,
}

a.spells["Cobra Shot"] = {
    ID = 77767,
}

a.spells["Aspect of the Hawk"] = {
    Type = "form",
    ID = 13165,
    NotWhileMoving = 1,
    CheckFirst = function()
        return not s.Form(20043) -- Aspect of the Wild
    end
}

a.spells["Explosive Trap"] = {
    ID = 13813,
}

a.spells["Kiroptyric Sigil"] = {
    Type = "item",
    ID = 77113,
    Continue = 1,
    FlashColor = "yellow",
    NoGCD = 1,
}

a.spells["Call Pet"] = {
    ID = 883,
    FlashID = { 883, 83242, 83243, 83244, 83245, 982 },
    FlashColor = "yellow",
    CheckFirst = function()
        return not IsMounted() and not UnitExists("pet")
    end
}

----------------------------------------------------------------- Beast Mastery
a.spells["The Beast Within"] = { ID = 34471 }
a.spells["Frenzy Effect"] = { ID = 19615 }

a.spells["Focus Fire"] = {
    ID = 82692,
    Buff = 82692,
    BuffUnit = "player",
    FlashColor = "yellow",
    CheckFirst = function()
        return s.BuffStack(c.GetID("Frenzy Effect"), "pet") == 5
            and not c.HasBuff("The Beast Within")
            and c.GetCooldown("Bestial Wrath") > 10
    end
}

a.spells["Kill Command"] = {
    ID = 34026,
    NoPowerCheck = 1,
    EvenIfNotUsable = 1,
    CheckFirst = function()
        return a.Focus >= s.SpellCost(34026)
    end
}

a.spells["Bestial Wrath"] = {
    ID = 19574,
    NoGCD = 1,
    FlashColor = "yellow",
    CheckFirst = function()
        return a.Focus >= 70
            and s.BuffStack(c.GetID("Frenzy Effect"), "pet") == 5
    end
}

a.spells["Fervor"] = {
    ID = 82726,
    FlashColor = "yellow",
    CheckFirst = function()
        return a.Focus < 50 and s.Power("pet") < 50
    end
}

------------------------------------------------------------------ Marksmanship
a.spells["Marked for Death"] = { ID = 1978 }
a.spells["Improved Steady Shot"] = { ID = 53220 }
a.spells["Autoshot"] = { ID = 75 }

local function underCarefulAim()
    return s.HealthPercent() > 90
        and c.HasTalent("Careful Aim")
        and not a.GetConfig("no_careful_aim")
end

a.spells["Serpent Sting unless Careful Aim"] = {
    ID = 1978,
    Continue = 1,
    FlashColor = "yellow",
    NoPowerCheck = 1,
    EvenIfNotUsable = 1,
    CheckFirst = function(z)
        return a.Focus >= s.SpellCost(z.ID)
            and not underCarefulAim()
            and c.ShouldCastToRefresh(
                "Serpent Sting", "Serpent Sting", 2, true, "Chimera Shot")
    end
}

a.spells["Chimera Shot"] = { 
    ID = 53209,
    TravelTimeEstimate = 1,
    NoPowerCheck = 1,
    EvenIfNotUsable = 1,
    CheckFirst = function(z)
        if a.Focus < s.SpellCost(z.ID) then
            return false
        end
        
        if not underCarefulAim() then
            return true
        end
        
        return c.HasTalent("Marked for Death")
            and s.DebuffDuration(markDebuffs)
                < c.GetBusyTime() + c.EstimateTravelTime("Chimera Shot")
    end
}

a.spells["Aimed Shot"] = {
    ID = 19434,
}

a.spells["Aimed Shot!"] = {
    ID = 82928,
    FlashID = { c.GetID("Aimed Shot"), 82928 },
    Override = function()
        return s.Buff(82926, "player", c.GetBusyTime())
    end
}

a.spells["Marksmanship Focus Dump"] = {
    NoPowerCheck = 1,
    EvenIfNotUsable = 1,
    CheckFirst = function(z)
        local aimedID = c.GetID("Aimed Shot")
        if underCarefulAim() then
            z.ID = aimedID
            return a.Focus >= s.SpellCost(z.ID)
        end
        
        local cast = c.GetCastTime("Aimed Shot")
        if cast < 1.9 then
            z.ID = aimedID
            return a.Focus >= s.SpellCost(z.ID)
        end
        
        z.ID = c.GetID("Arcane Shot")
        if a.Focus < s.SpellCost(z.ID) then
            return false
        end
        
        local _, regen = GetPowerRegen()
        local focus = a.Focus - s.SpellCost(z.ID) + cast * regen
        local duration = c.GetCooldown("Chimera Shot") - cast
        local needed = s.SpellCost(c.GetID("Chimera Shot"))
        local addedTime = math.max(1, c.GetCastTime("Steady Shot"))
        local addedFocus = a.FocusAdded() + regen * addedTime
        while true do
            if focus >= needed then
                return true
            elseif duration <= 0 then
                return false
            end
            duration = duration - addedTime
            focus = focus + addedFocus
        end
    end
}

a.spells["Rapid Fire"] = {
    ID = 3045,
}

a.spells["Rapid Fire if Talented"] = {
    ID = c.GetID("Rapid Fire"),
    FlashColor = "yellow",
    Continue = 1,
    CheckFirst = function()
        return c.HasTalent("Posthaste")
            and not c.HasBuff("Rapid Fire")
    end
}

a.spells["Readiness"] = {
    ID = 23989,
    FlashColor = "yellow",
    Continue = 1,
    CheckFirst = function()
        return c.GetCooldown("Rapid Fire") > 60
    end
}

a.spells["Kill Command under Resistance is Futile"] = {
    ID = c.GetID("Kill Command"),
    RequireBuff = 82897,
}

a.spells["Steady Shot"] = {
    ID = 56641,
}
c.RegisterAura("Steady Shot", "Improved Steady Shot")

local function nextSSIsImproved()
    if c.InfoMatches(c.GetQueuedInfo(), "Steady Shot") then
        return not a.NextSSIsImproved
    else
        return a.NextSSIsImproved
    end
end

local function shouldRefreshISS(shots, earlyRefresh)
    if c.IsCasting("Steady Shot") and not nextSSIsImproved() then
        return false
    else
	    return shots * c.GetCastTime("Steady Shot")
	        > c.GetBuffDuration("Improved Steady Shot") - earlyRefresh
	end
end

a.spells["Steady Shot No Buff"] = {
    ID = c.GetID("Steady Shot"),
    CheckFirst = function()
        return shouldRefreshISS(2, 0)
    end
}

a.spells["Steady Shot Refresh Buff"] = {
    ID = c.GetID("Steady Shot"),
    CheckFirst = function()
        return shouldRefreshISS(3, 2)
    end
}

a.spells["Steady Shot 2"] = {
    ID = c.GetID("Steady Shot"),
    CheckFirst = function()
        return nextSSIsImproved() and shouldRefreshISS(2, 2)
    end
}

---------------------------------------------------------------------- Survival
a.spells["Lock and Load"] = { ID = 56453 }

a.spells["Explosive Shot"] = {
    ID = 53301,
    NoPowerCheck = 1,
    EvenIfNotUsable = 1,
    CheckFirst = function()
        return a.Focus >= s.SpellCost(53301) or c.HasBuff("Lock and Load")
    end
}

a.spells["Black Arrow"] = {
    ID = 3674,
    Continue = 1,
    FlashColor = "yellow",
    NoPowerCheck = 1,
    EvenIfNotUsable = 1,
    CheckFirst = function()
        return a.Focus >= s.SpellCost(3674)
    end
}
